start cmd /k py frontiersman/server.py 127.0.0.1 1233
start cmd /k py frontiersman/ClientMain.py 127.0.0.1 1233  Zander 3
start cmd /k py frontiersman/ClientMain.py 127.0.0.1 1233  Bob 3
start cmd /k py frontiersman/ClientMain.py 127.0.0.1 1233  John 3